--[[
**********************************
Licence: GPL
(c) 2012, 2013 msl, rudi, sinter
v0.8.113f live osd 036pure
required CHDK 1.2 r.2453 or higher
**********************************
@title Twilight Live Pure

@param a aktuelles Datum?
@default a 1
@range a 0 1

@param y Jahr
@default y 1
@values y 2012 2013 2014 2015 2016

@param m Monat
@default m 0
@values m Jan Feb Mar Apr Mai Jun Jul Aug Sep Okt Nov Dez

@param d Tag
@default d 30
@range d 1 31

@param s Sommerzeit?
@default s 1
@range s 0 1

@param t Zeitzone in Min.?
@default t 14
@values t -660 -600 -540 -480 -420 -360 -300 -240 -210 -180 -150 -120 -60 0 60 120 180 210 240 270 300 330 345 360 390 420 480 540 57, 600 660 720 765

@param o 16:9-Anzeige?
@default o 0
@range o 0 1

@param x Speicher-Anzeige?
@default x 0
@range x 0 1

@param u Start with SUN-LIVE-OSD?
@default u 0
@values u Off On
]]

-- Parameter Rev. Pure 
z=0    --List Astronom. Twilight? =OFF
l=10   --StartDelay for draw msec =1000ms
b=1    --Break OSD by SHOOT_HALF? =ON
v=0    --OSD: Override Charts?    =OFF
w=1    --OSD: Acoustic Feedback?  =ON
p=2    --OSD: TimeStep +- by ZOOM =1h
r=0    --OSD: start Full/Reduced  =FULL
q=0    --OSD: BlueHour/Nautic     =BlueHour
n=0    --OSD: Event-NightColors   =UNI
c=0    --OSD: ColorProfile Grey   =NewCam


--------------------GLOBALS----------------------
local SCREEN_W = 360+(120*o) --+120px 16/9screen
SCREEN_H = 240
local WINDOW_B = (SCREEN_H-20)
local FONT_W = 8
local FONT_H = 16
TRANSP   = 256
BLACK    = 257
WHITE    = 258
RED      = 259
RED_LT   = 261
GREEN    = 262
GREEN_DK = 263
GREEN_LT = 264
BLUE     = 265
GREY     = 268
GREY_DK  = 269
GREY_LT  = 270
YELLOW   = 271
YELLOW_DK= 272
YELLOW_LT= 273

--gps false or gpsLib for valid cameras
gps = os.stat("A/CHDK/LUALIB/gpsLib.lua") and require("gpsLib")

local SCALE = imath.scale
local PI = imath.pi
local muldiv = imath.muldiv
local mul = imath.mul
local div = imath.div
local sind = imath.sind
local cosd = imath.cosd
local asind = imath.asind
local acosd = imath.acosd
local NOON = 12*SCALE
local DAY = 2*NOON

local sfmt = string.format
local tn   = tonumber
local os_d = os.date

--data
h1 =  -833 --sunset/sunrise h=-50'
h2 = -6000 --civil twilight h= -6
h4 =-12000 --nautical twilight h= -12
h5 =-18000 --astronomical Twilight h= -18
h3 =  6000 --golden hour    h=  6
DST = s --daylight save time
TZ_ID = t+1
local location = {}

--------------------GLOBAL FUNCTIONS-------------
--Split string & write in array -> string.split(string,delimiter)
function string:split(delimiter)
    local result, str = { },(string.gsub(self,delimiter,",")) --change delimiter to defined char ","
    for part in string.gmatch(str.. ",","([%s%w-]+)"..",") do result[#result+1] = part end --split on ","
    return result
end

--remove leading & trailing spaces -> string.trim(string)
function string:trim() return (string.gsub(self,"^%s*(.-)%s*$","%1")) end

--returns integer and fraction part
function dec(val,div) return val/div, math.abs(val%div) end

--decimal degrees to degrees and minutes
function dmsfmt(val,div)
    if type(val) == "number" then
        local h,r = dec(val,div)
        return h, (dec(r*60,div))
    elseif type(val) == "boolean" then
        if val then return "++","++" else return "--","--" end
    end
end

--date functions
function get_os_date() return tn(os_d("%d")), tn(os_d("%m")), tn(os_d("%Y")) end

function get_day_of_year(dd, mm, yyyy) return os_d("*t",os.time{year=yyyy, month=mm, day=dd})["yday"] end

function day_max(mm,yyyy) local dd = mm == 2 and 29 or 31 return get_day_of_year(1,mm+1,yyyy)==get_day_of_year(dd,mm,yyyy) and dd-1 or dd end

function trim_day(dd,mm,yyyy) local day_limit = day_max(mm,yyyy) return dd > day_limit and day_limit or dd end

--------------------FUNCTIONS--------------------
--sunrise/sunset/twilight calculation
function sun_times(lat,lon,h,doy,tz)
    --[[based on http://lexikon.astronomie.info/zeitgleichung/
    doy: Nummer des Tages im aktuellen Jahr
    h  : Winkel der Sonne ber dem horizont in Dezimalform * imath.scale
    lat: geografische Breite in Dezimalform * imath.scale   [-90000 < lat < 90000]
    lon: geografische Lnge in Dezimalform * imath.scale  [-180000 <= lon <= 180000]
    tz : Zeitzone in Dezimalform * imath.scale
    return: Sonnenaufgang, Sonnenuntergang, Sonnenhchststand in Dezimalform * imath.scale
        * Sonnenaufgang, Sonnenuntergang in Dezimalform * imath.scale
            oder false bei Polarnacht oder true bei Mitternachtssonne
        * Sonnenhchststand in Dezimalform * imath.scale]]
    --Zeitgleichung: WOZ - MOZ = -0.171*sind(deg(0.0337) * T + deg(0.465)) - 0.1299*sind(deg(0.01787) * T - deg(0.168))
    local WOZ,MOZ = mul(sind(1931*doy+26127), -171), mul(sind(1024*doy-9626), 130)
    --Hchststand = 12 + Zeitzone - geographische Lnge /15 - Zeitgleichung
    local top = (NOON+tz-div(lon, 15*SCALE)-(WOZ-MOZ)+DAY)%DAY
    --Deklination_deg = deg(0.4095)*sind(deg(0.016906)*(T-80.086))
    local D = muldiv(sind(muldiv(96864, doy*1000-80086,100000)), 23462, 1000)
    --Zeitdifferenz = 12*arccos((sin(h) - sin(B)*sin(Deklination)) / (cos(B)*cos(Deklination)))/Pi
    --Teil 1: (sin(h) - sin(B)*sin(Deklination)) / (cos(B)*cos(Deklination))
    local geo_ref = div((sind(h)-mul(sind(lat), sind(D))), mul(cosd(lat), cosd(D)))
    --Mitternachtssonne/Polarnacht => type="boolean"; true fr Mitternachtssonne
    local rising,setting = (geo_ref < 0),(geo_ref < 0)
    if math.abs(geo_ref) <= SCALE then
        --Teil2: Zeitdifferenz = 12*arccos(geo_ref)/Pi = 12*acosd(geo_ref)/180
        local time_diff = acosd(geo_ref)/15
        rising, setting = (top-time_diff+DAY)%DAY,(top+time_diff+DAY)%DAY
    end
    return rising,setting,top
end

--calculation angle of the sun for a given time and location
function sun_pos(lat,lon,doy,utc,tz)
    --[[based on http://lexikon.astronomie.info/zeitgleichung/
    doy: Nummer des Tages im aktuellen Jahr
    utc: Uhrzeit in Dezimalform * imath.scale oder false fr Hchststand
    lat: geografische Breite in Dezimalform * imath.scale   [-90000 < lat < 90000]
    lon: geografische Lnge in Dezimalform * imath.scale  [-180000 <= lon <= 180000]
    tz : Zeitzone in Dezimalform * imath.scale
    return: Winkel der Sonne ber dem horizont in Dezimalform * imath.scale, Uhrzeit des Hchststandes der Sonne]]
    local WOZ,MOZ = mul(sind(1931*doy+26127),-171), mul(sind(1024*doy-9626),130)
    local top = (NOON+tz-div(lon, 15*SCALE)-(WOZ-MOZ)+DAY)%DAY
    local topdelta = utc and utc-top or 0
    local D = muldiv(sind(muldiv(96864,doy*1000-80086,100000)),23462,1000)
    return asind(mul(cosd(topdelta*15),mul(cosd(lat),cosd(D)))+mul(sind(lat),sind(D))),top
end

--timezone functions
tz_tab = {-660,-600,-540,-480,-420,-360,-300,-240,-210,-180,-150,-120,-60,0,60,120,180,210,240,270,300,330,345,360,390,420,480,540,570,600,660,720,765}

function get_tz_index(tz_min)
    local res
    for i = 14,tz_min < 0 and 1 or #tz_tab, tz_min < 0 and -1 or 1 do if tz_tab[i] == tz_min then res = i break end end
    return res
end

function get_tz_value(tzi,dst) return ((dst == 1 and 60 or 0)+tz_tab[tzi])*100/6 end

function get_tz_str(tzi) return (tz_tab[tzi]==0 and "\177" or tz_tab[tzi] > 0 and "+" or "")..sfmt("%d:%02d",dec(tz_tab[tzi],60)) end

--dataset functions
function insert_dataset(datatab, pos)
    if #datatab == 4 then
        pos = pos or #location+1
        location[pos] = {}
        location[pos].name = datatab[1]
        location[pos].lat  = datatab[2]
        location[pos].lon  = datatab[3]
        location[pos].tz   = datatab[4]
    end
end

--read geo data from a file
function load_data(dfile, mode) --mode 0=overwrite, 1=append
    if os.stat(dfile) then
        local id = #location
        local id_start = id
        if mode == 0 then id = 0 end
        local file = io.open(dfile)
        for line in file:lines(dfile) do
            if not string.find(line,"#") then
                local array = string.split(line,";")
                if array and #array == 4 and type(array[1]) == "string" then
                    array[1] = string.trim(array[1])
                    array[2] = tn(array[2])
                    array[3] = tn(array[3])
                    array[4] = get_tz_index(tn(array[4]))
                    if (type(array[2]) == "number") and (array[2] < 900  or array[2] > -900)  and
                       (type(array[3]) == "number") and (array[3] <= 1800 or array[3] >= -1800) and
                       (type(array[4]) == "number") then
                        id = id+1
                        insert_dataset(array,id)
                    end
                end
            end
        end
        file:close()
        return true,id-id_start
    else
        return false
    end
end

--write data in user data file
function write_data(id)
    local user_data_file = "A/CHDK/GEODATA/GeoUserD.txt"
    local file
    if os.stat(user_data_file) then file = io.open(user_data_file,"ab") else file = io.open(user_data_file,"wb") end
    file:write(sfmt("%s;%d;%d;%d\n",location[id].name,location[id].lat,location[id].lon,tz_tab[location[id].tz]))
    file:close()
end

--delete data from array
function del_data(id) if #location >= id then table.remove(location,id) end end

--draw empty main frame with title1 & title2, keys & descriptions
function draw_main_frame(title1,title2,keys,description)
    draw_rect_filled(0,0,SCREEN_W,WINDOW_B,WHITE,BLACK,2)
    draw_string(SCREEN_W/2-#title1*FONT_W/2, 4,title1,WHITE,BLACK)
    draw_string(SCREEN_W/2-#title2*FONT_W/2,20,title2,WHITE,BLACK)
    if x == 1 then
        local mem = get_meminfo()
        draw_string(SCREEN_W-FONT_W*9,22,sfmt("M:%6d",mem.free_size),WHITE,BLACK)
    end
    draw_line(0, 37,SCREEN_W, 37,WHITE)
    draw_line(0,185,SCREEN_W,185,WHITE)
    draw_string(SCREEN_W/2-#keys*FONT_W/2,187, keys,BLUE,BLACK)
    draw_string(SCREEN_W/2-#description*FONT_W/2,202,description,WHITE,BLACK)
end

--select geo data set from array, returns id or 0 when canceled
function sel_data()
    local loop,res,id = true,0,location_id
    local str_title = "Auswahl Datensatz"
    local str_key1 = " [\18]  [SET]  [MENU]"
    local str_key2 = "Daten   OK   Abbruch"
    draw_main_frame("",str_title,str_key1,str_key2)
    while loop do
        local str_lat = sfmt("Lat: %d.%d ",dec(location[id].lat,10))
        local str_data1 = "Datensatz-Nr.: "..tostring(id)
        local str_data2 = location[id].name
        local str_data3 = str_lat..sfmt("Lon: %d.%d",dec(location[id].lon,10))
        draw_rect_filled(0+3,70,SCREEN_W-3,130,BLACK,BLACK)
        draw_string(SCREEN_W/2-#str_data1*FONT_W/2, 70,str_data1,WHITE,BLACK)
        draw_string(SCREEN_W/2-#str_data2*FONT_W/2, 90,str_data2,WHITE,BLACK)
        draw_string(SCREEN_W/2-#str_data3*FONT_W/2,110,str_data3,WHITE,BLACK)
        wait_click()
        if is_pressed("down") then bp(1) id = id==#location and 1 or id+1
        elseif is_pressed("up") then bp(1) id = id==1 and #location or id-1
        elseif is_pressed("set") then bp(1) res,loop = id,false
        elseif is_pressed("menu") then bp(1) loop = false
        end
    end
    return res
end

--submenu geo data
function menu_geo_data()
    local loop, ret = true
    local str_keys = {"[\24]", "[\25]", "[\26]", "[\27]", "[\23]", "[\29]", "[SET]", "[MENU]", "[DISP]"}
    local str_title = "Geo-Daten"
    local str_key1 = "Datenstze aus Datei laden"
    local str_key2 = "Datensatz eingeben"
    local str_key3 = "Datensatz editieren"
    local str_key4 = "Datensatz in Datei speichern"
    local str_key5 = "Datensatz lschen"
    local str_key6 = "GPS-Position hinzufgen"
    local str_key7 = "zurck"
    local str_key11 = "Datei-Browser wird geffnet."
    local str_key12 = "Whle eine Datei aus."
    local str_key13 = "Konnte keine externen Daten laden!"
    local str_key14 = "Datenstze eingelesen."
    while loop do
        draw_main_frame("",str_title,str_keys[8],str_key7)
        draw_string(SCREEN_W/10, 50,str_keys[7],BLUE,BLACK)
        draw_string(SCREEN_W/ 4, 50,str_key1,  WHITE,BLACK)
        draw_string(SCREEN_W/10, 70,str_keys[3],BLUE,BLACK)
        draw_string(SCREEN_W/ 4, 70,str_key2,  WHITE,BLACK)
        draw_string(SCREEN_W/10, 90,str_keys[4],BLUE,BLACK)
        draw_string(SCREEN_W/ 4, 90,str_key3,  WHITE,BLACK)
        draw_string(SCREEN_W/10,110,str_keys[1],BLUE,BLACK)
        draw_string(SCREEN_W/ 4,110,str_key4,  WHITE,BLACK)
        draw_string(SCREEN_W/10,130,str_keys[2],#location > 1 and BLUE or GREY,BLACK)
        draw_string(SCREEN_W/ 4,130,str_key5,  WHITE,BLACK)
        if gps then
            tgps = gps.data(10)
            draw_string(SCREEN_W/10,150,str_keys[9],tgps.status and BLUE or GREY,BLACK)
            draw_string(SCREEN_W/ 4,150,str_key6,WHITE,BLACK)
        end
        wait_click()
        if is_pressed("set") then
            bp(1)  draw_rect_filled(0+3,50,SCREEN_W-3,170,BLACK,BLACK)
            draw_string(SCREEN_W/2-#str_key11*FONT_W/2, 90,str_key11,WHITE,BLACK)
            draw_string(SCREEN_W/2-#str_key12*FONT_W/2,110,str_key12,WHITE,BLACK)
            sleep(1500)
            draw_clear()
            local data_file = file_browser("A/CHDK/GEODATA")
            draw_clear()
            draw_main_frame("",str_title,str_keys[8],str_key7)
            local data,count = 0
            if data_file ~= nil then
                data,count = load_data(data_file)
            else
                data = false
            end
            draw_rect_filled(0+3,50,SCREEN_W-3,170,BLACK,BLACK)
            if data == false then
                draw_string(SCREEN_W/2-#str_key13*FONT_W/2,90,str_key13,WHITE,BLACK)
            else
                draw_string(SCREEN_W/2-#sfmt("%d %s",count,str_key14)*FONT_W/2,90,sfmt("%d %s",count,str_key14),WHITE,BLACK)
            end
            sleep(1500)
            draw_rect_filled(0+3,50,SCREEN_W-3,170,BLACK,BLACK)
        elseif is_pressed("right") then bp(1) input_geo_data()
        elseif is_pressed("left") then
            bp(1)  ret = sel_data()
            if ret > 0 then input_geo_data(ret) end
        elseif is_pressed("up") then
            bp(1)  ret = sel_data()
            if ret > 0 then write_data(ret) end
        elseif is_pressed("down") then
            bp(1) 
            if #location > 1 then
                ret = sel_data()
                if ret > 0 then del_data(ret) end
            end
        elseif is_pressed("display") then
            bp(1) 
            if gps and tgps.status then
                input_geo_data(tgps.lat,tgps.lon,get_tz_index(tgps.tz),sfmt("GPS %2d:%02d",tgps.time[1],tgps.time[2]))
            end
        elseif is_pressed("menu") then bp(1) loop = false
        end
    end
end

--manual input geo data: edit exist entry use ID as arg, edit new entry use LAT, LON, TZ, NAME as args
function input_geo_data(...)
    local idx = 2
    local geo = {"Lat: ","+",0,0,".",0," Lon: ","+",0,0,0,".",0," TZ: ","      ",66,101,110,117,116,122,101,114,100,97,116,101,110}
    local x = SCREEN_W/2-33*FONT_W/2-15/2 --33 chars geo[1] - geo[15] 15x1px spacing
    local xx = x
    local loop = true
    local str_title = "manuelle Eingabe Geo-Daten"
    local str_key1  = "[\29]  [\18]  [ZOOM] [SET] [MENU]   "
    local str_key2  = "Pos Wert  10er    OK  Abbruch  "
    local tzi,id,arg1,arg2,arg3,arg4 = get_tz_index(0)
    if select("#", ...) == 4 then arg1,arg2,arg3,arg4 = ...
    elseif select("#", ...) == 1 then
        id = (...>=1 or ...<=#location) and ...
        if id then arg1,arg2,arg3,arg4 = location[id].lat,location[id].lon,location[id].tz,location[id].name end
    end
    if type(arg1) == "number" and type(arg2) == "number" and type(arg3) == "number" and type(arg4) == "string" and
        arg1 > -900 and arg1 < 900 and arg2 >= -1800 and arg2 <= 1800 and arg3 > 0 and arg3 <= #tz_tab then
        if arg1 < 0 then geo[2] = "-" arg1 = math.abs(arg1) end
        for i = 6, 3, -1 do if type(geo[i]) == "number" then arg1, geo[i] = dec(arg1, 10) end end
        if arg2 < 0 then geo[8] = "-" arg2 = math.abs(arg2) end
        for i = 13, 9, -1 do if type(geo[i]) == "number" then arg2, geo[i] = dec(arg2, 10) end end
        tzi = arg3
        for i = 1, 13 do geo[15+i] = string.byte(arg4, i) or 32 end
    end
    geo[15] = sfmt("%6s",get_tz_str(tzi))
    draw_main_frame("",str_title,str_key1,str_key2)
    draw_rect(xx-FONT_W,70,xx+FONT_W*35+2,125,WHITE)
    while loop do
        draw_rect_filled(xx,80,xx+FONT_W*33,115,BLACK,BLACK) --del data line
        for i = 1, 15 do --draw data
            draw_string(x,80,geo[i],WHITE,i == idx and RED or BLACK)
            x = x+#tostring(geo[i])*FONT_W+1 --+1px spacing
        end
        x = xx
        for i = 16, 28 do --draw location
            draw_string(x, 100, sfmt("%c", geo[i]), WHITE, i == idx and RED or BLACK)
            x = x+FONT_W+1 --+1px spacing
        end
        x = xx
        wait_click()
        if is_pressed("right") then
            bp(1) idx = idx+1 
            if idx == 5 or idx == 7 or idx == 12 or idx == 14 then idx = idx+1 end
            if idx > 28 then idx = 2 end
        elseif is_pressed("left") then
            bp(1) idx = idx-1
            if idx == 5 or idx == 7 or idx == 12 or idx == 14 then idx = idx-1 end
            if idx < 2 then idx = 28 end
        elseif is_pressed("up") then
            bp(1) 
            if idx == 2 or idx == 8 then geo[idx] = geo[idx]=="+" and "-" or "+"
            elseif idx == 3 then geo[idx] = geo[idx] < 8 and geo[idx]+1 or 0
            elseif idx == 9 then geo[idx] = geo[idx] == 1 and 0 or 1
            elseif idx == 15 then tzi = tzi<#tz_tab and tzi+1 or 1 geo[15] = sfmt("%6s",get_tz_str(tzi))
            elseif idx > 15 then geo[idx] = geo[idx]<122 and geo[idx]+1 or 32
            else geo[idx] = geo[idx]<9 and geo[idx]+1 or 0
            end
        elseif is_pressed("zoom_in") then
            bp(1)  if idx > 15 then geo[idx] = geo[idx]<=122-10 and geo[idx]+10 or 32 end
        elseif is_pressed("down") then
            bp(1) 
            if idx == 2 or idx == 8 then geo[idx] = geo[idx] == "+" and "-" or "+"
            elseif idx == 3 then geo[idx] = geo[idx]>0 and geo[idx]-1 or 8
            elseif idx == 9 then geo[idx] = geo[idx] == 1 and 0 or 1
            elseif idx == 15 then tzi = tzi>1 and tzi-1 or #tz_tab geo[15] = sfmt("%6s",get_tz_str(tzi))
            elseif idx > 15 then geo[idx] = geo[idx] > 32 and geo[idx]-1 or 122
            else geo[idx] = geo[idx]>0 and geo[idx]-1 or 9
            end
        elseif is_pressed("zoom_out") then
            bp(1)  if idx > 15 then geo[idx] = geo[idx]>=32+10 and geo[idx]-10 or 122 end
        elseif is_pressed("set") then
            bp(1) 
            local lat = geo[3]*100+geo[4]*10+geo[6]
            local lon = geo[9]*1000+geo[10]*100+geo[11]*10+geo[13]
            if lat < 900 and lon <= 1800 then
                if geo[2] == "-" then lat = -lat end
                if geo[8] == "-" then lon = -lon end
                local tz = tzi
                local str_location = ""
                for i = 16, 28 do str_location = str_location..sfmt("%c", geo[i]) end
                str_location = string.trim(str_location)
                insert_dataset({str_location,lat,lon,tz},id)
                location_id = id and id or #location
                loop = false
            else
                idx = 9
                for i = 1, idx-1 do x = x+#tostring(geo[i])*FONT_W+1 end --+1px spacing
                for i= idx, idx+4 do -- draw error
                    draw_string(x, 80, "!", WHITE, RED)
                    x = x+FONT_W+1
                end
                x = xx
                sleep(300)
            end
        elseif is_pressed("menu") then bp(1) loop = false
        end
    end
end

--main: charts
function select_chart()
    local loop = true
    local str_keys = {"[\24]","[\25]","[DISP]","[MENU]"}
    local str_title = "Diagramme"
    local str_key1 = "Sonnenstand fr einen Tag"
    local str_key2 = "Jahresdiagramm Auf/Untergang"
    local str_key3 = "Live-OSD"
    local str_key4 = "zurck"
    while loop do
        draw_main_frame("",str_title,str_keys[4],str_key4)
        draw_string(SCREEN_W/10, 70,str_keys[1],BLUE,BLACK)
        draw_string(SCREEN_W/ 4, 70,str_key1,  WHITE,BLACK)
        draw_string(SCREEN_W/10, 90,str_keys[2],BLUE,BLACK)
        draw_string(SCREEN_W/ 4, 90,str_key2,  WHITE,BLACK)
        draw_string(SCREEN_W/10,110,str_keys[3],BLUE,BLACK)
        draw_string(SCREEN_W/ 4,110,str_key3,  WHITE,BLACK)
        if (u ~= 1 and stop_now == false and leave_osd_by_MENU == false and v~= 1) then 
                wait_click() 
                if is_pressed("down") then bp(1) sun_chart()
                elseif is_pressed("up") then bp(1) sun_chart_high()
                elseif is_pressed("display") then bp(1) Sun_Live_OSD()
                elseif is_pressed("menu") then bp(1) loop = false
                end
            
            elseif leave_osd_by_MENU==true then leave_osd_by_MENU=false loop = false -- override this Menu if leaving OSD to swap location
            --elseif v == 1 then Sun_Live_OSD()
            else Sun_Live_OSD() u = 0

        end
        if stop_now == true then loop = false end
    end
end

--sunrise/sunset year chart for the current data set
function sun_chart()
    draw_rect_filled(0,0,SCREEN_W,WINDOW_B,WHITE,BLACK,2)
    for r = 1,11 do draw_line(3, WINDOW_B-r*WINDOW_B/12,SCREEN_W-3,WINDOW_B-r*WINDOW_B/12,GREY) end
    for c = 1,11 do draw_line(c*SCREEN_W/12,3,c*SCREEN_W/12,WINDOW_B-3,GREY) end
    draw_string(3,3,"24:00",WHITE,BLACK)
    draw_string(3,WINDOW_B-3-FONT_H,"0:00/Jan",WHITE,BLACK)
    draw_string(SCREEN_W-3-(FONT_W*3),WINDOW_B-3-FONT_H,"Dez",WHITE,BLACK)
    for d = 1,365 do
        local sunrise,sunset = sun_times(Lat,Lon,h1,d,Tz)
        if type(sunrise)=="number" then
            draw_pixel(3+d*(SCREEN_W-6)/365,WINDOW_B-sunrise*WINDOW_B/24000,GREEN)
            draw_pixel(3+d*(SCREEN_W-6)/365,WINDOW_B- sunset*WINDOW_B/24000,RED)
        end
    end
    wait_click()
end

--sun chart of a day
function sun_chart_high()
    local h
    draw_rect_filled(0,0,SCREEN_W,WINDOW_B,WHITE,BLACK,2)
    for r = 1,17 do draw_line(3,WINDOW_B-r*WINDOW_B/2/9,SCREEN_W-3,WINDOW_B-r*WINDOW_B/2/9,r == 9 and WHITE or GREY) end
    for c = 1,11 do draw_line(c*SCREEN_W/12, 3, c*SCREEN_W/12,WINDOW_B-3,GREY) end
    draw_string(3,3,"90",WHITE,BLACK)
    draw_string(3,WINDOW_B-3-FONT_H,"-90/00:00",WHITE,BLACK)
    draw_string(SCREEN_W-3-(FONT_W*5),WINDOW_B-3-FONT_H,"24:00",WHITE,BLACK)
    local step = 1000/60*((24000/(1000/60))/SCREEN_W)
    for t = 0,24000,step do h = sun_pos(Lat,Lon,DoY,t,Tz) draw_pixel(3+t*(SCREEN_W-6)/24000,(h/1000*-1)*11/9+WINDOW_B/2,YELLOW_LT) end
    wait_click()
end

--calendar for date selection
function calendar()
    local function day_of_week(dd,mm,yyyy)
        local dw = os_d("*t",os.time{year=yyyy,month=mm,day=dd})["wday"]-1
        return dw == 0 and 7 or dw
    end

    local function add_month(val,mm,yyyy)
        mm = mm+val%12
        if mm > 12 then yyyy, mm = yyyy+1, mm-12
        elseif mm < 1 then yyyy,mm = yyyy-1,mm+12
        end
        return mm, yyyy
    end

    local function next_day(val)
        day=day+val
        if day > day_max(month,year) then day = day-day_max(month, year) month,year = add_month(1,month,year) end
        if day < 1 then month,year = add_month(-1,month,year) day = day_max(month,year)+day end
    end

    local function next_month(val)
        month, year = add_month(val,month,year)
        day = trim_day(day,month,year)
    end

    local function make_month_table(mm,yyyy)
        local days = {}
        local week_day = day_of_week(1,mm,yyyy)
        if week_day > 1 then for i = 1, week_day-1 do table.insert(days,i,"  ") end end
        for i = 1, day_max(mm, yyyy) do table.insert(days, i-1+week_day,sfmt("%02d",i)) end
        if #days < 37 then for i=#days+1,37 do table.insert(days,i,"  ") end end
        return days,week_day
    end

    local str_month = {"Januar","Februar","Mrz","April","Mai","Juni","Juli","August","September","Oktober","November","Dezember"}
    local D,M,Y = day,month,year
    local str_title
    local str_key1 = "[\29]  [\23]  [ZOOM] [DISP] [SET] [MENU]"
    local str_key2 = "Tag Woche Monat  Heute   OK   Abruch"
    local str_days = "Mo  Di  Mi  Do  Fr  Sa  So"
    local x = SCREEN_W/2-#str_days *FONT_W/2
    local xl= #str_days*FONT_W
    local xx= x
    local y = 50
    local yy= y
    local days,offset
    local loop,tmonth = true,0

    while loop do
        x = xx
        y = yy
        if tmonth~=month then
            str_title = sfmt("%s %4d", str_month[tn(month)],year)
            draw_main_frame("",str_title,str_key1,str_key2)
            draw_rect_filled(x-4,y-2,x+xl+2,y+FONT_H+1,GREY,GREY)
            draw_rect_filled(x+xl-FONT_W*6-2,y+18,x+xl+2,y+18+17*6,YELLOW_DK,YELLOW_DK)
            draw_string(x,y,str_days,WHITE,GREY)
            days,offset = make_month_table(month,year)
            tmonth = month 
        end
        y = y+22
        for i = 1,#days do
            col = i ~= (day+offset-1) and ((i%7 == 6 or i%7 == 0) and YELLOW_DK or BLACK) or RED
            draw_string(x, y, days[i], WHITE,col)
            x = x+FONT_W*4
            if i%7 == 0 then x = xx y = y+17 end
        end
        wait_click()
        if is_pressed("right") then bp(1) next_day(1)
        elseif is_pressed("down") then bp(1) next_day(7)
        elseif is_pressed("zoom_in") then bp(1) next_month(1)
        elseif is_pressed("left") then bp(1) next_day(-1)
        elseif is_pressed("up") then bp(1) next_day(-7)
        elseif is_pressed("zoom_out") then bp(1) next_month(-1)
        elseif is_pressed("display") then bp(1) day, month, year = get_os_date()
        elseif is_pressed("set") then bp(1) loop = false
        elseif is_pressed("menu") then bp(1) day,month,year = D,M,Y loop = false
        end
    end
end

--informations for the recent day
function info_screen()
    local day,month,year = get_os_date()
    local DoY = get_day_of_year(day,month,year)
    local sunrise,sunset = sun_times(Lat,Lon,h1,DoY,Tz)
    local str_title = "Informationen"
    local str_key1  = "[MENU]"
    local str_key2  = "zurck"
    local loop = true
    draw_main_frame("",str_title,str_key1,str_key2)
    while loop do
        local minute = tn(os_d("%M"))
        local hour = tn(os_d("%H"))
        local t = hour*SCALE+minute*SCALE/60
        local td_rise = type(sunrise)=="number" and sunrise-t or sunrise
        local td_set = type(sunset)=="number" and sunset-t or sunset
        if type(td_rise) == "number" and td_rise <= 0 or type(tdrise) == "number" and td_set <= 0 then
            sunrise,sunset = sun_times(Lat,Lon,h1,DoY+1,Tz)
            if type(sunrise) == "number" then if td_rise <= 0 then td_rise = sunrise+DAY-t end else td_rise = sunrise end
            if type(sunset) == "number" then if td_set <=0 then td_set = sunset+DAY-t end else td_set = sunset end
        end
        local str_time1 = string.gsub(sfmt("%2d.%02d.%04d %2d:%02d Uhr",day,month,year,hour,minute),"0","O")
        local str_time2 = string.gsub(sfmt("nchster Sonnenaufgang   in %2s:%02s Stunden",dmsfmt(td_rise,SCALE)),"0","O")
        local str_time3 = string.gsub(sfmt("nchster Sonnenuntergang in %2s:%02s Stunden",dmsfmt(td_set,SCALE)),"0","O")
        local sun_max,time_max = sun_pos(Lat,Lon,DoY,false,Tz)
        local hh,mm = dmsfmt(sun_max,1000)
        local str_sun1 = string.gsub(sfmt("Sonnenstand: %3d%02d'",dmsfmt(sun_pos(Lat,Lon,DoY,hour*1000+minute*1000/60,Tz),1000)),"0","O")
        local str_sun2 = string.gsub(sfmt("Hchststand: %3d%02d' um %2d:%02d",hh,mm,dmsfmt(time_max,1000)),"0","O")
        draw_string(SCREEN_W/20, 60,str_time1,WHITE,BLACK)
        draw_string(SCREEN_W/20, 85,str_sun1, WHITE,BLACK)
        draw_string(SCREEN_W/20,105,str_sun2, WHITE,BLACK)
        draw_string(SCREEN_W/20,130,str_time2,WHITE,BLACK)
        draw_string(SCREEN_W/20,150,str_time3,WHITE,BLACK)
        wait_click(30000)
        if is_pressed("menu") or is_pressed("shoot_half") or is_pressed("display") or is_pressed("set") then bp(1) loop = false end
    end
end

--main screen: script starts here
function main_screen()
    local str_key1 = "[\18]"
    local str_key2 = "[SET]   [\27]   [\26]   [DISP]   [Halb]   [MENU]"
    local str_insert=" Chart " if v ==1 then str_insert="LiveOSD" end
    local str_key3 = "GeoData Cal   DST  "..str_insert.."   Info      End "
    local loop = true


    while loop do
        DoY = get_day_of_year(day,month,year)
        Tz = get_tz_value(location[location_id].tz,DST)
        Lat = location[location_id].lat
        local Lat_str = sfmt("%d.%d",dec(Lat,10))
        Lon = location[location_id].lon
        local Lon_str = sfmt("%d.%d",dec(Lon,10))
        Lat,Lon = Lat*100,Lon*100
        local sunrise, sunset, suntop = sun_times(Lat,Lon,h1,DoY,Tz)
        local dawn, dusk              = sun_times(Lat,Lon,h2,DoY,Tz)
        local gold_end, gold_start    = sun_times(Lat,Lon,h3,DoY,Tz)
        local naut_dawn, naut_dusk    = sun_times(Lat,Lon,h4,DoY,Tz)
        local astr_dawn, astr_dusk    = sun_times(Lat,Lon,h5,DoY,Tz)
        local str_0 = string.gsub(sfmt(" (%d) ",location_id),"0","O")
        local str_1 = string.gsub(sfmt("%s  TZ: %s",location[location_id].name,get_tz_str(location[location_id].tz)),"0","O")
        local str_2 = string.gsub(sfmt("%s %s %02d.%02d.%4d",Lat_str,Lon_str,day,month,year),"0","O")
        local str_res = {}
        sun_max, time_max = sun_pos(Lat,Lon,DoY,false,Tz)
        str_res[1] = string.gsub(sfmt("nauti. Morgendmmerung: %2s:%02s Uhr",dmsfmt(naut_dawn,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h4, DoY, naut_dawn)),"0","O")
        str_res[2] = string.gsub(sfmt("brgl. Morgendmmerung: %2s:%02s Uhr",dmsfmt(dawn,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h2, DoY, dawn)),"0","O")
        str_res[3] = string.gsub(sfmt("         Sonnenaufgang: %2s:%02s Uhr",dmsfmt(sunrise,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h1, DoY, sunrise)),"0","O")
        str_res[4] = string.gsub(sfmt("   Ende Goldene Stunde: %2s:%02s Uhr",dmsfmt(gold_end,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h3, DoY, gold_end)),"0","O")
--        str_res[5] = string.gsub(sfmt("     Sonnenhchststand: %2s:%02s    ",dmsfmt(suntop,SCALE))..string.format(" aprx_%3s",azi_string(Lat, Lon, sun_max, DoY, time_max)),"0","O")
        str_res[5] = string.gsub(sfmt("     Sonnenhchststand: %2s:%02s Uhr",dmsfmt(suntop,SCALE))..string.format("    %3s",azi_string(Lat, Lon, sun_max, DoY, time_max)),"0","O")
        str_res[6] = string.gsub(sfmt(" Beginn Goldene Stunde: %2s:%02s Uhr",dmsfmt(gold_start,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h3, DoY, gold_start)),"0","O")
        str_res[7] = string.gsub(sfmt("       Sonnenuntergang: %2s:%02s Uhr",dmsfmt(sunset,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h1, DoY, sunset)),"0","O")
        str_res[8] = string.gsub(sfmt("brgl.  Abenddmmerung: %2s:%02s Uhr",dmsfmt(dusk,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h2, DoY, dusk)),"0","O")
        str_res[9] = string.gsub(sfmt("nauti.  Abenddmmerung: %2s:%02s Uhr",dmsfmt(naut_dusk,SCALE))..string.format("    %3s",azi_string(Lat, Lon, h4, DoY, naut_dusk)),"0","O")
        if z == 1 then -- zusaetzliche Einblendung Astron. Twilight
            str_res[1] = string.gsub(sfmt("ast. %2s:%02s",dmsfmt(astr_dawn,SCALE))..string.format(" %3s\124 naut.",azi_string(Lat, Lon, h5, DoY, astr_dawn)),"0","O")..string.sub(str_res[1],23,string.len(str_res[1]))
            korrigierter_wert = azi_string(Lat, Lon, h5, DoY, astr_dusk)
            --if azi_string(Lat, Lon, h5, DoY, astr_dawn)=azi_string(Lat, Lon, h5, DoY, astr_dusk) then korrigierter_wert=360-azi_string(Lat, Lon, h5, DoY, astr_dusk)
            str_res[9] = string.gsub(sfmt("ast. %2s:%02s",dmsfmt(astr_dusk,SCALE))..string.format(" %3s\124 naut.",korrigierter_wert),"0","O")..string.sub(str_res[9],23,string.len(str_res[9]))
        end
        draw_main_frame("","",str_key2,str_key3)
        draw_string(10,4,str_key1,BLUE,BLACK)
        draw_string(10+FONT_W*#str_key1,4,str_0,WHITE,BLACK)
        draw_string(10+FONT_W*#str_key1+FONT_W*#str_0,4,str_1,WHITE,BLACK)
        if DST == 1 then draw_string(SCREEN_W-FONT_W-10,4,"\15",YELLOW_LT,BLACK) draw_string(SCREEN_W/2-#str_key3*FONT_W/2+13*8-2,202,"\15",YELLOW_LT,BLACK)end
        draw_string(10+FONT_W*#str_key1+FONT_W*#str_0,20,str_2,WHITE,BLACK)
        draw_string(SCREEN_W-(60*o)-(4+(7*8)),20,"\124 Azim.",WHITE,BLACK)--Sinter
        for i = 1,9 do draw_string(SCREEN_W/2-#str_res[i]*FONT_W/2,24+16*i,str_res[i],i == 5 and YELLOW_LT or WHITE,BLACK) end
        draw_string((60*o)+38*8-2,24+16*5+5,"\126",YELLOW_LT,BLACK) -- tiefergesetzte Tilde fr "ungefaehr" bei Azimut des Zenits
        --[[if DST ==1 then 
            for i = 1,9 do draw_string((60*o)+31*8,24+16*i,"DST",YELLOW_LT,BLACK) end
            --if z==1 then 
            --    draw_string((60*o)+11*8,24+16*1,"\15",YELLOW_LT,BLACK)
            --    draw_string((60*o)+11*8,24+16*9,"\15",YELLOW_LT,BLACK)
            --end
        end  ]]

        
        --draw_string(80,100,tostring(DST).." "..tostring(astr_dusk),BLACK,WHITE) --for bugtrace azimuth!!!
        if (u ~= 1 and stop_now == false) then wait_click()
                if is_pressed("down") then bp(1) location_id = location_id==#location and 1 or location_id+1
                elseif is_pressed("up") then bp(1) location_id = location_id == 1 and #location or location_id-1
                elseif is_pressed("right") then bp(1) DST = DST == 0 and 1 or 0
                elseif is_pressed("set") then bp(1) menu_geo_data() if location_id > #location then location_id = 1 end
                elseif is_pressed("left") then bp(1) calendar()
                elseif is_pressed("display") then bp(1) leave_osd_by_MENU = false select_chart()
                elseif is_pressed("shoot_half") then bp(1) info_screen()
                elseif is_pressed("menu") then bp(1) loop = false
                end
            else select_chart()
        end
        if stop_now == true then loop = false end
    end
end

------SINTER
--calculation azimuth
function azimuth(lat, lon, h, doy, utc) --returns rounded Azimut in Deg 
    --deklination + zeitgleichung(in min.)
    local D = muldiv(sind(muldiv(96864,doy*1000-80086,100000)),23462,1000)
    local WOZ, MOZ = mul(sind(1931*doy+26127),-171), mul(sind(1024*doy-9626),130)
    --based on http://www.jgiesen.de/SME/tk/index.htm
    --azimuth y = cos(azimut) = -[sin(lat)*sin(h) - sin(deklin)] / [cos(lat)*sin(arccos(sin(h)))]
    -- Division durch 0 abfangen; Divisor dann mit kleinstmoeglichem Wert "1" besetzten. Vorzeichen koennte noch fehlerhaft sein
    divi=mul(cosd(lat),sind(acosd(sind(h)))) --Divisor
    if divi==0 then divi=1 end --hilfsweise Eintausendstel statt Null
    local azimuth = acosd(div(mul(sind(lat), sind(h))-sind(D),divi)*-1)
    if utc-(Tz)+1000 > NOON+div(15*SCALE-lon,15*SCALE)-div((WOZ-MOZ)*60,60*SCALE) then azimuth=360*SCALE-azimuth end
    utc_global=utc  -- hilfsweise Uebergabe fuer adder
    if DST==1 and utc>0 and utc<1000 then --wenn Sommerzeit dann Fehlerkorrektur fr 00:00 bis 01:00 Uhr. Evtl. besser >= statt > etc.; Evtl Bedingung DST noch entfernen   
        return (360-(azimuth+500)/1000)
    else return (azimuth+500)/1000 --"+500" sorgt automatisch fr Rundung des Werts
    end
end

function azi_string(lat, lon, h, doy, utc)  --liefert Azimutwert als Ziffern-String oder als "---" falls kein Azi existent
    if type(utc) ~= "boolean" then  --  "Falls nicht Mitternachtssonne etc." dann...
        return tostring(azimuth(lat, lon, h, doy, utc))
    else return "---"  --Falls Mitternachtssonne etc.
    end
end

function bp(rep)
    for i=1,rep do
        if i>1 then sleep(100) end
        if w ==1 then play_sound(4) end
    end
end

--displays live sun events
function Sun_Live_OSD()
    --bp(1)
    leave_osd_by_MENU=false
    local loop = true
    local local_time = "C"
    local swap_order = 1
    local nightmulticolor = n
    local color = GREEN_LT
    if c == 1 then GREY,GREY_DK = GREY_DK,GREY  c=0 end -- swap grey once; "c=0": "no further swap if function recalled"
    local full = r == 0 and true or false
    local loc_str = string.sub(location[location_id].name,1,12) --max 12 chars
    local year, month, day, hour, minute, hours, current_h, zenit_eventtime, sun_y,
          oneminute_x, oneminute_y, fiveminute_x, fiveminute_y, zenit_rel_x, zenit_rel_y,
          deep_rel_x, deep_rel_y, angle_h_rel_x, angle_h_rel_y,
          sun_min, sun_max, time_max, time_min, angle_h_str,
          rising, setting, visual_sunrise, visual_sunset, gold_morning, gold_evening, blue_morning, blue_evening,
          az, x_azimut_rel, y_azimut_rel, shadow_rel
    local dial_m_x = SCREEN_W/2
    local dial_m_y = SCREEN_H/2
    local out_out_r = 100
    local out_in_r  =  80
    local out_plate_r = (out_out_r-out_in_r)/2
    local out_5min_r = 3
    local out_1min_r = 1
    local in_out_r = 56
    local in_in_r  = 50
    local in_plate_r = (in_out_r-in_in_r)/2
    local in_in_out_r = in_in_r-2 -- -2 wg. Trennlinie dazwischen
    local in_in_in_r  = 42
    local in_5min_r = 1
    local wall_x = SCREEN_W-10 -- "Wand" X-Position
    local horizont_y = 60 -- "Horizont"  Y-Position
    local sun_r = 4
    local cover_r = 6
    local plusday = 0
    local compass_x = SCREEN_W-35
    local compass_y = 160
    local compass_r = 20
    local add_incr
    local timebg -- Background Zeiteinblendung (rot falls Simulation mittels adder gestartet)

    local function draw_event_plate(h_angle,times,txt_cl,plate_cl) --"times" i.S.v. Eventcountdown
        h_angle=string.gsub(h_angle,"0","O")
        local x, y, flat
        if times<1000 and times>=0 then --Event innerhalb der nchsten Stunde uerer Uhrenkranz
            x, y = imath.recd((out_in_r+1+out_plate_r)*SCALE,(times*60*(360/60))-90000) --Platzierung Eventscheibe als relative Koordinaten
            --Next 3 Lines: Garantiere dem bevorstehenden Event die dominierende Ueberlappung (Teil 1):
            if times<33 then -- Speichere Daten eines Ereignisses der naechsten 2 Minuten fuer ein spaetestes redraw.
                redraw_flag=true re_x,re_y=x,y re_h_angle=h_angle re_txt_cl=txt_cl re_plate_cl=plate_cl
            end
            --Eventscheibe abflachen falls Scheibe zwischen 58-Minuten- und 12-Uhr-Position zwecks Unterscheidung 12-Uhr-Position
            if times>=(SCALE-(SCALE*(60-58)/60)) then flat=3 else flat=0 end --Hhenradius der Scheibe um 3 px abflachen  
            draw_ellipse_filled(dial_m_x+(x*9/8000),dial_m_y+(y/SCALE)+(flat/2),out_plate_r*9/8,out_plate_r-flat, plate_cl,txt_cl) --flachere Scheibe tiefer platzieren
            draw_string(dial_m_x+(x*9/8000)-8, dial_m_y+(y/SCALE)-8+1+(flat/2),h_angle,txt_cl,plate_cl)
        end
        if times<NOON and times>=0 then --in den nchsten 12 Stunden, innerer Uhrenkranz, uere Bahn
            x, y = imath.recd((in_in_r+in_plate_r)*SCALE, (times*(360/12))-90000) --Eventscheibe als relative Koordinaten
            draw_ellipse_filled(dial_m_x+(x*9/8000),dial_m_y+(y/SCALE),in_plate_r*9/8,in_plate_r,plate_cl,txt_cl)
        end
        if times<DAY and times>=NOON then --in 12 bis 24 Stunden, innerer Uhrenkranz, innere Bahn
            x, y = imath.recd((in_in_in_r+in_plate_r)*SCALE, (times*(360/12))-90000) --Eventscheibe als relative Koordinaten
            draw_ellipse_filled(dial_m_x+(x*9/8000),dial_m_y+(y/SCALE),in_plate_r*9/8,in_plate_r,plate_cl,txt_cl)
        end
    end

    local function draw_str_round_bg(x,y,str,r,txt_cl,bg_cl)
        str=string.gsub(str,"0","O")
        draw_ellipse_filled(x         ,y+7,r,8,bg_cl,bg_cl)
        draw_ellipse_filled(x+8*#str-1,y+7,r,8,bg_cl,bg_cl)
        draw_line(x,y-1,x+8*#str-1,y-1,bg_cl)
        draw_string(x,y,str,txt_cl,bg_cl)
    end

    local function draw_rect_filled_rounded(x1,y1,x2,y2,r,cl)
        local f=r*9/8
        draw_rect_filled(x1+f,y1  ,x2-f,y2-r  ,cl,cl)
        draw_rect_filled(x1  ,y1+r,x2  ,y2-r-r,cl,cl)
        draw_ellipse_filled(x1+f,y1+r  ,f,r,cl,cl)
        draw_ellipse_filled(x1+f,y2-r-r,f,r,cl,cl)
        draw_ellipse_filled(x2-f,y1+r  ,f,r,cl,cl)
        draw_ellipse_filled(x2-f,y2-r-r,f,r,cl,cl)
    end
    
    local function draw_star(x,y,clr)
        draw_ellipse_filled(x,y,3,6,RED_LT)
        draw_line(x-3,y-5,x+3,y+5,clr)
        draw_line(x-3,y+5,x+3,y-5,clr)
        draw_line(x  ,y-5,x  ,y+5,clr)
        draw_line(x-3,y  ,x+3,y  ,clr)
    end

    local function help(cl1,cl2)
        local x = 5+(60*o)
        draw_clear()
        draw_rect_filled(0,0,SCREEN_W,238,cl1,cl1)
        draw_rect_filled_rounded(0,0,SCREEN_W,238,16,cl2)
        draw_str_round_bg(18+8,  2, "SINTERS SUN-LIVE-OSD",12,cl1,YELLOW_LT)
        draw_string(SCREEN_W-(16*8)-18,2,"Exit: press MENU",RED,cl2)
        draw_line(5,19,SCREEN_W-5,19,cl1)
        draw_string(x,   21,"MENU   Exit Sun-Live-OSD  \124   ZOOM  time +/-",cl1,cl2)
        draw_string(x,   37,"SET    SWAP Camera-Time (C) / Local-Time (L)"   ,cl1,cl2)
        draw_string(x,   53,"DOWN   SWAP Blue-Hour / Nautical-Twilight"      ,cl1,cl2)
        draw_string(x,   69,"LEFT   SWAP ColorProfile: OldCam / NewCam"      ,cl1,cl2)
        draw_string(x,   85,"RIGHT  Color Nightevents: uni- / multicolor"    ,cl1,cl2)
        draw_string(x,  101,"DISP   SWAP Full Live-OSD / Reduced Live-OSD"   ,cl1,cl2)
        --nachfolgende Reihenfolge belassen fuer Zeilenueberlappung
        draw_string(x+8,207,"       SHADOWMETER:   Curr. shadow-length"      ,cl1,cl2)
        draw_string(x  ,192,"        AZIMUTHMETER:  Direction of sun"        ,cl1,cl2)
        draw_string(x  ,177,"Right:  RANGEMETER:    Max/Min/Curr. elevat."   ,cl1,cl2)
        draw_string(x  ,162,"Left:   BLUEHOURMETER: Blue-Hour-Elevation"     ,cl1,cl2)
        draw_string(x  ,147," \183Small outer ring: next 12h / inner: 12-24h",cl1,cl2)
        draw_string(x  ,132," \183Large ring: Events during next 6O minutes" ,cl1,cl2)
        draw_string(x  ,117,"Centre: EVENTMETER:    Progress of elevation"   ,cl1,cl2) --Grenze
        draw_line(5,116,SCREEN_W-5,116,cl1) 
        draw_str_round_bg(SCREEN_W/2-20,224,"<ALT>",8,cl2,RED)
        if b == 1 then draw_str_round_bg(SCREEN_W-(17*8),224,"Break: SHOOTHALF",8,cl1,RED_LT) end
        wait_click()
        bp(1)
    end
    
    adder=0
    if p == 0 then add_incr=250
        elseif p==1 then add_incr=500
        elseif p==2 then add_incr=1000
        elseif p==3 then add_incr=2000
        elseif p==4 then add_incr=3000
        elseif p==5 then add_incr=6000
        elseif p==6 then add_incr=12000
    end

    while loop do
        redraw_flag=false
        day, month, year = get_os_date()
        hour    = tn(os_d("%H"))
        minute  = tn(os_d("%M"))
        second  = tn(os_d("%S"))
        hours = hour*SCALE+minute*SCALE/60+second*SCALE/3600+adder--aktuelle utc
        --[SET] local/camera time, displays L or C
        if local_time == "L" then
            hours = hours-get_tz_value(TZ_ID, DST)+Tz
            if hours>DAY then hours=hours-DAY day=day+1 end
            if hours<0 then hours=hours+DAY day=day-1 end
        end
        current_h, zenit_eventtime = sun_pos(Lat,Lon,DoY,hours,Tz)
        draw_clear()
        -- ALT rounded bg:
        draw_str_round_bg(SCREEN_W/2-20,224,"<ALT>",8,WHITE,RED)
        --GRAPH links
        draw_rect_filled(0,10,5,59,WHITE,WHITE) --Weies Rechteck als Grundflche >0 bzw 5
        --darin ein nach oben auslaufendes rotes Dreieck 0 bis5 
        for i=0,6 do
            draw_line(i,59,0,25,RED)
        end
        --GOLDEN HOUR
        --Skalierung des roten Bereichs (0 bis +6): 2-Pixel-Linie auf jeder der 6 korrekt definierten Grad-Stufen
        for i=1,6 do
            draw_line(0,60-(i*6)  ,6,60-(i*6)  ,BLACK)  
            draw_line(0,60-(i*6)+1,6,60-(i*6)+1,BLACK)
        end
        if full then
            draw_str_round_bg(18,24-7,"6",8,WHITE,RED)
            draw_line(11  ,24  ,15+1,24  ,WHITE) -- Pluszeichen
            draw_line(13  ,24-3,13  ,24+3,WHITE) -- Pluszeichen
            draw_line(13+1,24-3,13+1,24+3,WHITE) -- Pluszeichen
        end
        --Schwarze Umrandung des Bereichs:
        draw_line(0,10,6,10,BLACK)
        draw_line(6,10,6,59,BLACK)
        --BlUE HOUR & NIGHT:
        --"Grad-Stufen" -1 bis -6 fr die Intensitt der Dmmerung
        --unten ein kleiner auslaufender grner (vormals schwarzer) Bereich unterhalb der -6 Dmmerung
        --Blaue-Stunde-Flche komplett einfrben
        draw_rect_filled(0,60,5,179,GREY_DK,GREY_DK)
        --Blaue Stunde Dreiecksverlauf
        for i=0,5 do --6. Feld nun symmetrisch zu nautic ebenfalls teilfarben
            draw_line(i,60,i,60+((i+1)*20/(q+1)),BLUE)
            if q == 1 then draw_line(i,180,i,180-10-(10*i),BLACK) end --zusaetzlicher Schwarzkeil falls nautische Daemmerung gewaehlt
        end
        --Blaue-Stunde Skalierung 3px stark + Gradzahl + optional nautische Zwischenskalierung 1px stark
        for i=0,(6*(q+1)),(q+1) do
            draw_rect_filled(0,60+(i*20/(q+1))+1,6,60+(i*20/(q+1))-1,WHITE,WHITE)
            if q==1 then for i=1,12-1,2 do draw_line(0,60+(i*10),6,60+(i*10),WHITE,WHITE) end end --Zwischenskalierung
            if full then
                draw_str_round_bg(18,60+(i*20/(q+1))-7,sfmt("%d",i),8,WHITE,BLUE)
                if i>0 then draw_line(11,60+(i*20/(q+1)),15+1,60+(i*20/(q+1)),WHITE) end--Minuszeichen
            end
        end
        draw_rect_filled(0,60+15-(8*q),5,60+17-(8*q),RED,RED) -- Markierung fr "Visual Sunrise/Sunset"
        draw_rect_filled(0,180+2,6,200  ,GREEN_LT,GREEN_LT) --Nachtbereich unterhalb Dmmerung
        draw_rect_filled(0,180+2,1,200  ,WHITE   ,WHITE) --Nachtbereich zustzlich optisch abheben
        draw_line(6,180,6,200,WHITE) --Rahmen rechts fr Nachtbereich
        draw_line(0,200,6,200,WHITE) --Rahmen unten fr Nachtbereich
        draw_line(6, 60,6,200,WHITE) --rechte Abgrenzung des Blaue-Stunde- und Nacht-Bereichs
        draw_line(6, 10,6, 59,BLACK) --rechte Abgrenzung des wei-roten Tag-Bereichs
        draw_line(7, 10,7,200,WHITE) --rechte Abgrenzung gesamt zustzlich
        --YELLOW SUN
        if     current_h<=0 and current_h>= (-6500*(q+1))  then sun_y=60+(-(current_h*20/(q+1))/SCALE)
        elseif current_h<(-6500*(q+1))            then sun_y=194 --Nacht Sonne < -6,5 Einrasten in Nachtindikator
        elseif current_h>(0) and current_h<= 6500 then sun_y=60-((current_h*6)/SCALE) --Tag rtliche Sonne zw. 0 u. +6,5 ber Horizont
        elseif current_h>6500                     then sun_y=16 --Tag Sonne > +6.5 ber Horizont Einrasten in Tagesindikator
        end
        --needle+sun+dec_value
        for i=-2,2 do
            draw_line(3,sun_y+i,31,sun_y+i,(i<-1 or i>1) and BLACK or YELLOW_LT)-- attempt to perform arithmetic on local sun_y (a nil value); warum?; wsl. Lsg: YELLOW SUN angepasst
        end
        draw_ellipse_filled(3,sun_y,5,5,BLACK    ,BLACK) --Rand
        draw_ellipse_filled(3,sun_y,3,3,YELLOW_LT,BLACK) --Flche
        if full then --FULL OSD
            draw_str_round_bg(18,sun_y-8,sfmt("%s%d.%d",current_h<0 and "-" or "",dec((math.abs(current_h)+50)/100,10)),8,BLACK,YELLOW_LT) --nadelmitwandernd; "+50" -> Wert auf erste Dezimale gerundet
            --Hhenwinkel als Grad/Minute ber 12-Uhr-Position
            angle_h_str=string.gsub(sfmt("%s%s%02s'",current_h<0 and "-" or "",dmsfmt(math.abs(current_h),1000)),"0","O")
            draw_ellipse_filled(dial_m_x-(#angle_h_str*8)/2, dial_m_y-out_out_r-16+18,16,18,WHITE, WHITE)
            draw_ellipse_filled(dial_m_x+(#angle_h_str*8)/2-1, dial_m_y-out_out_r-16+18,16,18,WHITE, WHITE)
            draw_string(dial_m_x-(#angle_h_str*8)/2, dial_m_y-out_out_r-16,angle_h_str, BLACK, WHITE)
            --CLOCK_FACE:
            --Aussenkranz zeichnen:
            draw_ellipse_filled(dial_m_x,dial_m_y,(out_out_r+2)*9/8  ,out_out_r+2,WHITE  ,WHITE) --Rundrahmen um Aussenkranz
            draw_ellipse_filled(dial_m_x,dial_m_y, out_out_r   *9/8  ,out_out_r  ,GREY,WHITE)
            draw_ellipse_filled(dial_m_x,dial_m_y, out_in_r    *9/8-1,out_in_r-1 ,TRANSP ,TRANSP)
            draw_ellipse(dial_m_x, dial_m_y,out_in_r*9/8-1,out_in_r-1,BLACK) --Auenkranz nach innen mit schwarzer Kreislinie abheben
            --Dessen 1-Minuten-Markierungen einzeichnen:
            for i=0,59 do
                oneminute_x,oneminute_y=imath.recd((out_out_r-2-out_1min_r)*SCALE,(i*360)/60*SCALE)
                draw_ellipse_filled(dial_m_x+oneminute_x*9/8000,dial_m_y+oneminute_y/SCALE,out_1min_r*9/8,out_1min_r,BLACK,BLACK) 
            end
            --Dessen 5-Minuten-Markierungen einzeichnen:
            for i=0,11 do
                fiveminute_x,fiveminute_y=imath.recd((out_out_r-1-out_5min_r)*SCALE,(i*360)/12*SCALE)
                draw_ellipse_filled(dial_m_x+(fiveminute_x*9/8000), dial_m_y+(fiveminute_y/SCALE),out_5min_r*9/8,out_5min_r,BLACK,BLACK) 
            end
            --usseren Innenkranz zeichnen:
            draw_ellipse_filled(dial_m_x, dial_m_y,in_out_r*9/8,in_out_r,GREY,GREY)
            draw_ellipse(dial_m_x, dial_m_y,in_out_r*9/8+1,in_out_r+1,BLACK) --mit uerer Kreislinie gegenber Hintergrund abheben
            --Inneren Innenkranz zeichnen:
            draw_ellipse_filled(dial_m_x, dial_m_y,in_in_out_r*9/8  ,in_in_out_r ,GREY_LT,GREY_LT)
            draw_ellipse_filled(dial_m_x, dial_m_y,in_in_in_r *9/8-2,in_in_in_r-2,TRANSP ,TRANSP)
            --Dessen Stunden-Markierungen (entspricht fiveminutemarks) werden erst nach dem Zeichnen der Eventscheiben eingezeichnet damit sie nicht berdeckt werden
            draw_ellipse(dial_m_x, dial_m_y,in_in_in_r*9/8-2-1,in_in_in_r-2-1,BLACK) --mit innerer Kreislinie gegenber Hintergrund abheben
            --Deckblatt der "Drehachse" der Zeiger zeichnen:
            draw_ellipse_filled(dial_m_x,dial_m_y, cover_r     *9/8,cover_r     ,BLACK,BLACK)
            draw_ellipse_filled(dial_m_x,dial_m_y, cover_r*8/10*9/8,cover_r*8/10,WHITE,WHITE)
            draw_ellipse_filled(dial_m_x,dial_m_y, cover_r*5/10*9/8,cover_r*5/10,BLACK,BLACK)
            --Uhrzeit (Kamera- oder Ortszeit) unter der Zeigerachse einblenden:
            if adder==0 then timebg=WHITE else timebg=RED_LT end
            draw_str_round_bg(dial_m_x-28,dial_m_y +cover_r+3,sfmt("%s %02d:%02d",local_time,hours/SCALE,(hours%SCALE*60)/SCALE),8,BLACK,timebg)
            if DST ==1 then draw_star(dial_m_x-2*8-1,dial_m_y+15,YELLOW_LT) end 
            --Location
            draw_str_round_bg(SCREEN_W-9-(FONT_W*#loc_str),205,loc_str,8,BLACK,WHITE)
            --aktuelle Schattenlnge als rel. Faktor der Hhe eines Krpers
            shadow_rel=(1000*SCALE)/(imath.tand(current_h))
            if (shadow_rel<(999*SCALE) and current_h>0) then shadow_str=sfmt("Shadow h\215%d.%02d",dec(shadow_rel/10,100)) else shadow_str="Shadow->inf." end
            draw_str_round_bg(SCREEN_W-9-(FONT_W*#shadow_str),224,shadow_str,8,BLACK,WHITE)
            --ZENIT
            sun_max, time_max = sun_pos(Lat,Lon,DoY,false,Tz)
            --Tiefststandberechnung:
            time_min = time_max+NOON --Annahme, Tiefststand ist exakt 12 Stunden nach Zenit
            --falls Tiefststand nach Mitternacht Zeitvariable um 24 h rckrechnen, Berechnungstag +1
            if time_min > DAY then time_min=time_min-DAY plusday = 1 else plusday = 0 end
            sun_min = sun_pos(Lat, Lon, DoY+plusday, time_min, Tz)
            --WALL
            for i=0,3 do
                draw_line(wall_x+i,horizont_y+30,wall_x+i,horizont_y-30,(i<1 or i>2) and BLACK or WHITE)
            end
            --HORIZONT
            for i=-2,2 do
                draw_line(wall_x-40,horizont_y-i,wall_x,horizont_y-i,(i<-1 or i>1) and BLACK or WHITE)
            end
            --oberer Zenitstrahl+Wert
            zenit_rel_x, zenit_rel_y = imath.recd(50*SCALE,sun_max) --Koordinaten
            draw_str_round_bg(math.min(SCREEN_W-32,wall_x-(zenit_rel_x*9/8000)-18),horizont_y-(zenit_rel_y/SCALE)-10,string.gsub(sfmt("%d",sun_max/SCALE),"0","O"),8,RED,WHITE)
            for i=-1,1 do
                draw_line(wall_x,horizont_y+i,wall_x-(zenit_rel_x*9/8000),horizont_y+i-(zenit_rel_y/SCALE),RED)
            end
            --unterer Zenitstrahl+Wert
            deep_rel_x, deep_rel_y = imath.recd(50*SCALE,sun_min)
            draw_str_round_bg(math.min(SCREEN_W-40,wall_x-(deep_rel_x*9/8000)+0),horizont_y-(deep_rel_y/SCALE)+3,string.gsub(sfmt("%d",sun_min/SCALE),"0","O"),8,RED,WHITE)
            for i=-1,1 do
                draw_line(wall_x,horizont_y+i,wall_x-(deep_rel_x*9/8000),horizont_y+i-(deep_rel_y/SCALE),RED)
            end
            --Hoehenwinkelstrahl+Sonne
            angle_h_rel_x, angle_h_rel_y = imath.recd(40*SCALE,current_h)
            for i=-2,2 do
                draw_line(wall_x,horizont_y+i,wall_x-(angle_h_rel_x*9/8000),horizont_y+i-(angle_h_rel_y/SCALE),(i<-1 or i>1) and BLACK or YELLOW_LT)
            end
            draw_ellipse_filled(wall_x-(angle_h_rel_x*9/8000),horizont_y-(angle_h_rel_y/SCALE),(sun_r+2)*9/8,sun_r+2,BLACK,BLACK) -- schwarze Umrandung der Sonnenscheibe
            draw_ellipse_filled(wall_x-(angle_h_rel_x*9/8000),horizont_y-(angle_h_rel_y/SCALE), sun_r   *9/8,sun_r  ,YELLOW_LT,BLACK) -- Sonnenscheibe
            --Events
            swap_order=swap_order*(-1) --each loop: swap order of layer of plates BH and VS
            for eventday = 1,-1,-1 do --0=today, 1=tomorrow Reihenfolge invers fr richtige Folge
                --Eventkategorie "alle weitere positiv ganzzahlige Hoehen-Gradzahlen"
                sun_max, time_max = sun_pos(Lat, Lon,DoY+eventday,false,Tz) --Zenit
                for angle=7,(sun_max/SCALE) do -- "angle" i.S.v. "event_angle"
                    rising,setting = sun_times(Lat,Lon,angle*SCALE,DoY+eventday,Tz)
                    if type(rising) =="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),rising +(eventday*DAY)-hours,BLACK,WHITE) end
                    if type(setting)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),setting+(eventday*DAY)-hours,BLACK,WHITE) end
                end
                --Eventkategorie Nacht: "alle weitere negativ ganzzahlige Hoehen-Gradzahlen"
                if nightmulticolor==1 then
                    if     eventday < 0 then color = GREEN_DK
                    elseif eventday > 0 then color = GREEN
                    else                     color = GREEN_LT
                    end
                end
                for angle=(-7-(6*q)),sun_min/SCALE,-1 do
                    rising,setting = sun_times(Lat,Lon,angle*SCALE,DoY+eventday,Tz)
                    if type(rising) =="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),rising +(eventday*DAY)-hours,BLACK,color) end
                    if type(setting)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),setting+(eventday*DAY)-hours,BLACK,color) end
                end
                --Visual Sunrise/Sunset
                    visual_sunrise,visual_sunset = sun_times(Lat,Lon,h1,DoY+eventday,Tz) --Lichtbrechung beruecksichtigt
                    if type(visual_sunrise)=="number" then draw_event_plate("VS",visual_sunrise+(eventday*DAY)-hours,YELLOW_LT,RED) end
                    if type(visual_sunset) =="number" then draw_event_plate("VS",visual_sunset +(eventday*DAY)-hours,YELLOW_LT,RED) end
                --Golden Hour
                for angle = 1,6 do
                    gold_morning,gold_evening = sun_times(Lat,Lon,angle*SCALE,DoY+eventday,Tz)
                    if type(gold_morning)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),gold_morning+(eventday*DAY)-hours,WHITE,RED) end
                    if type(gold_evening)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),gold_evening+(eventday*DAY)-hours,WHITE,RED) end
                end
                --Blue Hour
                blue_morning,blue_evening = sun_times(Lat,Lon,0,DoY+eventday,Tz) --Sonne am Horizont
                if type(blue_morning)=="number" then draw_event_plate("00",blue_morning+(eventday*DAY)-hours,WHITE,BLUE) end
                if type(blue_evening)=="number" then draw_event_plate("00",blue_evening+(eventday*DAY)-hours,WHITE,BLUE) end
                for angle =-1,(-6-(6*q)),-1 do
                    blue_morning,blue_evening = sun_times(Lat,Lon,angle*SCALE,DoY+eventday,Tz)
                    if type(blue_morning)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),blue_morning+(eventday*DAY)-hours,WHITE,BLUE) end
                    if type(blue_evening)=="number" then draw_event_plate(sfmt("%02d",math.abs(angle)),blue_evening+(eventday*DAY)-hours,WHITE,BLUE) end
                end
                --each loop: swap order of plates BH and VS -- wre eleganter mittels functions loesbar, aber vorerst wg. Uebersichtlichkeit bitte belassen
                if swap_order == 1 then
                  -- again: Now Visual Sunrise/Sunset-Plate in front of Blue-Hour-Plate
                    visual_sunrise,visual_sunset = sun_times(Lat,Lon,h1,DoY+eventday,Tz) --Lichtbrechung beruecksichtigt
                    if type(visual_sunrise)=="number" then draw_event_plate("VS",visual_sunrise+(eventday*DAY)-hours,YELLOW_LT,RED) end
                    if type(visual_sunset) =="number" then draw_event_plate("VS",visual_sunset +(eventday*DAY)-hours,YELLOW_LT,RED) end
                end
                --Astron. Daemmerung
                if q == 1 then
                        astro_twi_morning,astro_twi_evening = sun_times(Lat,Lon,-18*SCALE,DoY+eventday,Tz)
                        if type(astro_twi_morning)=="number" then draw_event_plate("As",astro_twi_morning+(eventday*DAY)-hours,BLACK,GREY_DK) end
                        if type(astro_twi_evening)=="number" then draw_event_plate("As",astro_twi_evening+(eventday*DAY)-hours,BLACK,GREY_DK) end
                end
                --Zenit
                sun_max,time_max = sun_pos(Lat,Lon, DoY+eventday,false,Tz)
                draw_event_plate("Ze",eventday*DAY+time_max-hours,BLACK,YELLOW_LT)
                --Tiefststand  (Nherungswert)  @MSL: Tiefststand bitte noch nicht code-optimieren da Algor. noch nicht gesichert
                time_min = time_max+NOON  --Unterstellung: der Tiefststand ist exakt 12 Stunden nach Zenit erreicht
                    --falls Sonnentiefststand nach Mitternacht Zeitvariable um 24 h rckrechnen, Berechnungstag +1
                    if time_min > DAY then time_min=time_min-DAY plusday = 1 else plusday = 0 end
                    sun_min = sun_pos(Lat, Lon, DoY+eventday+plusday, time_min, Tz)
                --draw_event_plate(sun_min/SCALE,eventday*DAY+plusday+time_min-hours    ,RED,BLACK)
                draw_event_plate("Lo",eventday*DAY+plusday+time_min-hours    ,WHITE,BLACK)
                --draw_event_plate(sun_min/SCALE,plusday*DAY+DAY-time_min-hours,RED,BLACK)
                --if eventday == 0 then draw_event_plate(sun_min/SCALE,time_max-12000-hours,RED,BLACK) end -- kurz nach Mitternacht muss auch TiefstEvent des "Vortags" angezeigt werden falls Zenit erst nach 12:00 Uhr
            end
            --Next if-Block: Garantiere dem bevorstehnden Event die dominierende Ueberlappung (Teil 2): redraw gespeichertes Event
            if redraw_flag==true then
                draw_ellipse_filled(dial_m_x+(re_x*9/8000),dial_m_y+(re_y/SCALE),out_plate_r*9/8,out_plate_r, re_plate_cl,re_txt_cl)
                draw_string(dial_m_x+(re_x*9/8000)-8, dial_m_y+(re_y/SCALE)-8+1,re_h_angle,re_txt_cl,re_plate_cl)
                redraw_flag=false
            end
            --Stunden-Markierungen Innenkranz (wie fiveminutemarks)
            for i=0,11 do
                fiveminute_x,fiveminute_y=imath.recd((in_in_out_r+in_5min_r)*SCALE,(i*360)/12*SCALE)
                draw_ellipse_filled(dial_m_x+(fiveminute_x*9/8000),dial_m_y+(fiveminute_y/SCALE),in_5min_r*9/8,in_5min_r,BLACK,BLACK) 
            end
            --ueren und inneren Innenkranz optisch trennen:
            draw_ellipse(dial_m_x, dial_m_y,(in_in_out_r+1)*9/8,in_in_out_r+1,BLACK)
            --ZEIGER
            for i=0,7 do
                draw_line(dial_m_x,dial_m_y-50,dial_m_x-i,dial_m_y-40,WHITE)   --l.o.klein
                draw_line(dial_m_x,dial_m_y-50,dial_m_x+i,dial_m_y-40,GREY_DK) --r.o.k
                draw_line(dial_m_x,dial_m_y   ,dial_m_x-i,dial_m_y-40,GREY)    --l.u.k
                draw_line(dial_m_x,dial_m_y   ,dial_m_x+i,dial_m_y-40,BLACK)   --r.u.k
                draw_line(dial_m_x,dial_m_y-90,dial_m_x-i,dial_m_y-70,WHITE)   --l.o.gro
                draw_line(dial_m_x,dial_m_y-90,dial_m_x+i,dial_m_y-70,GREY_DK) --r.o.g
                draw_line(dial_m_x,dial_m_y-56,dial_m_x-i,dial_m_y-70,GREY)    --l.u.g
                draw_line(dial_m_x,dial_m_y-56,dial_m_x+i,dial_m_y-70,BLACK)   --l.u.g
            end
            for i=-1,1,2 do
                draw_line(dial_m_x,dial_m_y-50,dial_m_x+7*i,dial_m_y-40,BLACK) --k
                draw_line(dial_m_x,dial_m_y   ,dial_m_x+7*i,dial_m_y-40,BLACK) --k
                draw_line(dial_m_x,dial_m_y-90,dial_m_x+7*i,dial_m_y-70,BLACK) --g
                draw_line(dial_m_x,dial_m_y-56,dial_m_x+7*i,dial_m_y-70,BLACK) --g
                draw_line(dial_m_x,dial_m_y   ,dial_m_x+7*i,dial_m_y-70,BLACK) --g
            end
            --Kompass fuer Azimut
            az = azimuth(Lat,Lon,current_h,DoY,hours)*SCALE
            draw_ellipse_filled(compass_x,compass_y,(compass_r+2)*9/8,compass_r+2,BLACK  ,BLACK) --Kompass
            draw_ellipse_filled(compass_x,compass_y, compass_r   *9/8,compass_r  ,WHITE,BLACK)
            draw_line(compass_x-compass_r, compass_y,compass_x+compass_r, compass_y,BLACK) --EastWestLine
            draw_line(compass_x, compass_y-compass_r,compass_x, compass_y+compass_r,BLACK) --NorthSouthLine
            draw_string(compass_x-4+1,compass_y-20+2,"N",BLACK,WHITE) --Norden kennzeichnen
            draw_string(compass_x+1-(#(tostring(az/SCALE))*8/2),compass_y+2,string.gsub(tostring(az/SCALE),"0","O"),BLACK,WHITE) --Azimutwert
            draw_ellipse(compass_x+1+(#(tostring(az/SCALE))*8/2)+3,compass_y+2+3,2,2,BLACK)
            draw_ellipse(compass_x+1+(#(tostring(az/SCALE))*8/2)+3,compass_y+2+3,1,2,BLACK)
            x_azimut_rel, y_azimut_rel = imath.recd((compass_r+sun_r)*SCALE, (az)-90000) --Koordinaten Sonne
            draw_ellipse_filled(compass_x+x_azimut_rel*9/8000,compass_y+(y_azimut_rel/SCALE),(sun_r+2)*9/8,sun_r+2,BLACK    ,BLACK) --Sonne
            draw_ellipse_filled(compass_x+x_azimut_rel*9/8000,compass_y+(y_azimut_rel/SCALE), sun_r   *9/8,sun_r  ,YELLOW_LT,BLACK)
            --help:
            draw_str_round_bg(8,206,"Help: UP",8,WHITE,BLACK)
        end
        wait_click(10000)
        if     is_pressed("right")   then bp(1) if nightmulticolor == 1 then nightmulticolor=0 n=0 color=GREEN_LT else nightmulticolor=1 n=1 end
        elseif is_pressed("display") then bp(1) if full == true then full=false r=1 else full=true r=0 end
        elseif is_pressed("down")    then bp(1) if q == 1 then q=0 else q=1 end
        elseif is_pressed("set")     then bp(1) if local_time == "C" then local_time = "L" else local_time = "C" end
        elseif is_pressed("left")    then bp(1) GREY,GREY_DK = GREY_DK,GREY
        elseif is_pressed("up")      then bp(1) help(BLACK,WHITE)
        elseif is_pressed("zoom_in") then bp(1) if utc_global<24000-add_incr then adder=adder+add_incr end  -- unbekannt: welche Auswirkung bei aktivem DST:
        elseif is_pressed("zoom_out") then bp(1) if utc_global>0+add_incr then adder=adder-add_incr end  --  Evtl. muessen Entscheidungsgrenzen dann um eine Stunde verschoben werden.
        elseif is_pressed("menu")    then bp(1) draw_clear() loop = false leave_osd_by_MENU=true
        elseif (is_pressed("shoot_half") and b == 1) then bp(2) stop_now = true loop = false
        end
    end
end
------/SINTER

function restore()
    cls()
    set_console_layout(0, 0, 25, 5)
    set_console_autoredraw(1)
end

--------------------MAIN-----------------------
if get_mode() == true then set_record(false) while get_mode() do sleep(10) end end
bp(1)
--load user data from file or Berlin as default
if not load_data("A/CHDK/GEODATA/GeoUserD.txt", 0) then insert_dataset({"Berlin",525,135,get_tz_index(60)}) end
location_id = 1

--date
year  = a == 1 and tn(os_d("%Y")) or 2012+y
month = a == 1 and tn(os_d("%m")) or m+1
day   = a == 1 and tn(os_d("%d")) or trim_day(d,month,year)

--user interface
cls()
set_console_layout(0, 0, 0, 0)
set_console_autoredraw(0)
delay = l*100
sleep(delay) -- time for draw
stop_now=false --init
main_screen()
restore()
